<?php

namespace App\DataTables\backend;

use App\Models\Site;
use Illuminate\Database\Eloquent\Builder as QueryBuilder;
use Yajra\DataTables\EloquentDataTable;
use Yajra\DataTables\Html\Builder as HtmlBuilder;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Html\Column;
use Yajra\DataTables\Html\Editor\Editor;
use Yajra\DataTables\Html\Editor\Fields;
use Yajra\DataTables\Services\DataTable;

class SitesDataTable extends DataTable
{
    /**
     * Build the DataTable class.
     *
     * @param QueryBuilder $query Results from query() method.
     */
    public function dataTable(QueryBuilder $query): EloquentDataTable
    {
        return (new EloquentDataTable($query))
            ->addColumn('#', function () {
                static $rowNumber = 0; // Static variable to maintain state
                return ++$rowNumber;   // Increment and return the row number
            })
            ->addColumn('action', function ($site) {
                return '<a data-id="' . $site->id . '" class="btn btn-sm btn-primary edit-site-btn"><i class="bi bi-pencil-square"></i></a>
                        <a data-id="' . $site->id . '" class="btn btn-sm btn-danger delete-site-btn"><i class="bi bi-trash3-fill"></i></a>';
            })
            ->setRowId('id');
    }

    /**
     * Get the query source of dataTable.
     */
    public function query(Site $model): QueryBuilder
    {
        return $model->newQuery()
            ->select([
                'sites.id', // Include the primary key
                'sites.name as site_name', // Rename the 'name' column to avoid conflicts
                'sites.company_id', // Include foreign keys if needed
                'sites.airport_id',
            ])
            ->with(['airport','company']);
    }

    /**
     * Optional method if you want to use the html builder.
     */
    public function html(): HtmlBuilder
    {
        return $this->builder()
            ->setTableId('sites-table')
            ->columns($this->getColumns())
            ->minifiedAjax()
            //->dom('Bfrtip')
            ->orderBy(1)
            ->selectStyleSingle()
            ->addTableClass('table table-bordered table-hover dataTable dtr-inline')
            ->buttons([
                Button::make('excel'),
                Button::make('csv'),
                Button::make('pdf'),
                Button::make('print')
            ]);
    }

    /**
     * Get the dataTable columns definition.
     */
    public function getColumns(): array
    {
        return [
            Column::make('#'),
            Column::make('site_name'),
            Column::make('company.name'),
            Column::make('airport.name'),
            Column::computed('action')
                ->exportable(false)
                ->printable(false)
                ->width(60)
                ->addClass('text-center')
        ];
    }

    /**
     * Get the filename for export.
     */
    protected function filename(): string
    {
        return 'Sites_' . date('YmdHis');
    }
}
