<?php

namespace App\Http\Controllers;

use App\Models\Payment;
use App\Notifications\BookingConfirmationEmail;
use App\Services\BookingService;
use App\Services\CustomerService;
use App\Services\PaymentService;
use Illuminate\Http\Request;
use App\Services\StripeService;
use Illuminate\Notifications\Notification;
use Illuminate\Support\Facades\Log;
use Stripe\Checkout\Session;
use Stripe\Exception\SignatureVerificationException;
use Stripe\Exception\UnexpectedValueException;
use Stripe\Stripe;

class PaymentController extends Controller
{
    protected $stripeService;
    private PaymentService $paymentService;
    private BookingService $bookingService;
    private CustomerService $customerService;

    public function __construct(StripeService   $stripeService, PaymentService $paymentService,
                                CustomerService $customerService,
                                BookingService  $bookingService
    )
    {
        $this->stripeService = $stripeService;
        $this->paymentService = $paymentService;
        $this->bookingService = $bookingService;
        $this->customerService = $customerService;
    }

    public function createCustomer(Request $request)
    {
        $customer = $this->stripeService->createCustomer($request->email, $request->name);
        return response()->json($customer);
    }

    public function createPaymentIntent(Request $request)
    {
        $intent = $this->stripeService->webhookData()->createPaymentIntent($request->amount, 'gbp');
        return response()->json($intent);
    }

    public function createCheckoutSession(Request $request)
    {
        $payment = $request->all();
        unset($payment['_token']);

        $data = [
            'payment_amount' => $request->payment_amount * 100,
            'payment_currency' => $request->payment_currency,
            'booking_id' => $payment['booking_id']
        ];
        // create stripe payment session
        $session = $this->stripeService->createPaymentIntent($data);

        // save payment
        $payment['session_id'] = $session->id;
        $this->paymentService->savePayment($payment);

        return redirect($session->url);
    }


    public function handle(Request $request)
    {
        // Retrieve Stripe signature from the header
        $signature = $request->header('Stripe-Signature');
        $endpoint_secret = config('services.stripe.webhook_secret');

        try {
            // Verify the webhook signature
            $event = $this->stripeService->webhookData($request->getContent(), $signature, $endpoint_secret);
            //$event = Webhook::constructEvent($request->getContent(), $signature, $endpoint_secret);

        } catch (UnexpectedValueException $e) {
            Log::error('Invalid Stripe payload', ['error' => $e->getMessage()]);
            return response()->json(['error' => 'Invalid payload'], 400);
        } catch (SignatureVerificationException $e) {
            Log::error('Stripe signature verification failed', ['error' => $e->getMessage()]);
            return response()->json(['error' => 'Signature verification failed'], 400);
        }

        // Handle different event types
        switch ($event->type) {
            case 'checkout.session.completed':
                $event = $event->data->object;
                $paymentId = $event->payment_intent;
                $paymentStatus = $event->payment_status;
                $customerEmail = $event->customer_details->email ?? null;

                $paymentIntent['payment_id'] = $paymentId;
                $paymentIntent['payment_log'] = $event;
                $this->paymentService->saveCheckoutSuccessLog($paymentIntent);

                $payment = $this->paymentService->getPaymentBySession('session_id', $event->id);

                if ($payment) {
                    $data = [
                        'payment_id' => $paymentId,
                        'payment_status' => $paymentStatus,
                        'customer_email' => $customerEmail,
                    ];
                    $this->paymentService->updatePayment($payment, $data);
                }
                //Log::info('Payment succeeded', ['payment_intent' => $paymentIntent]);

                $customer = $this->customerService->getCustomerById($payment->customer_id);

                $booking = $this->bookingService->formatBookingResponse($payment->booking_id);
                $booking['system_logo'] = asset('storage/images/logo/logo.svg');

                $customer->notify((new BookingConfirmationEmail($booking)));

                break;

            case 'charge.updated':
                //Log::info('Payment succeeded', ['charge_updated' => $event->data->object]);
                $event = $event->data->object;
                $eventPaymentMethod = $event->payment_method_details;
                $paymentId = $event->payment_intent;

                $payment = $this->paymentService->getPaymentBySession('payment_id', $paymentId);
                // ToDo : check not saving
                if ($payment && $eventPaymentMethod->card) {
                    $data = [
                        'card_brand' => $eventPaymentMethod->card->brand,
                        'card_funding' => $eventPaymentMethod->card->funding,
                        'card_exp_month' => $eventPaymentMethod->card->exp_month,
                        'card_exp_year' => $eventPaymentMethod->card->exp_year,
                        'card_last4' => $eventPaymentMethod->card->last4,
                        'receipt_url' => $event->receipt_url,
                    ];
                    $this->paymentService->updatePayment($payment, $data);
                }

                break;
            case 'payment_intent.failed':
                $paymentIntent = $event->data->object;

                $this->paymentService->saveCheckoutSuccessLog($paymentIntent);
                Log::warning('Payment failed', ['payment_intent' => $paymentIntent]);
                // Handle failed payment
                break;

            default:
                Log::info('Unhandled Stripe event', ['event' => $event->type]);
        }

        return response()->json(['message' => 'Webhook received']);
    }

    public function checkoutSuccess()
    {
        $sessionId = request()->query('session_id');
        //$this->paymentService->updatePayment(); // webhook
        return view('checkout-success', compact('sessionId'));

//        return response()->json(['message' => 'Payment is successful', 'session' => $sessionId ?? null ]);
//        $this->paymentService->updatePayment()
    }

    public function checkoutCancel()
    {
        return "Payment Canceled!";
    }

    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        //
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     */
    public function show(Payment $payment)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Payment $payment)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, Payment $payment)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Payment $payment)
    {
        //
    }
}
