<?php

namespace App\Http\Controllers\backend;

use App\DataTables\backend\ProductFeaturesDataTable;
use App\Http\Controllers\Controller;
use App\Models\ProductFeature;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;

class ProductFeatureController extends Controller
{
    public function __construct(private ProductFeature $model) { }

    /**
     * Display a listing of the resource.
     */
    public function index(ProductFeaturesDataTable $dataTable)
    {
        $data = ['title' => 'Configure Product Feature Information'];
        return $dataTable->render('backend.product_feature', $data);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        try {
            $request->validate(ProductFeature::$rules);

            // Handle file upload
            $iconPath = null;
            if ($request->hasFile('icon')) {
                $iconPath = $request->file('icon')->store('product_feature', 'public');
            }

            // Save to database
            $productFeature = $this->model->create([
                'title' => $request->title,
                'description' => $request->description,
                'icon' => $iconPath,
            ]);

            // Return success response
            return response()->json([
                'message' => 'Product feature saved successfully!',
                'product' => $productFeature, // Optionally return the saved product
            ], 200); // HTTP 200 OK

        } catch (\Illuminate\Validation\ValidationException $e) {
            // Handle validation errors
            return response()->json([
                'message' => 'Validation failed.',
                'errors' => $e->errors(),
            ], 422); // HTTP 422 Unprocessable Entity

        } catch (\Exception $e) {
            // Log the error for debugging
            Log::error('Error saving product feature: ' . $e->getMessage());

            // Return a generic error message
            return response()->json([
                'message' => 'An error occurred while saving the product feature. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        try {
            // Fetch the company data by ID
            $productFeature = $this->model->findOrFail($id);

            // Return the company data as JSON
            return response()->json($productFeature);

        } catch (\Exception $e) {
            // Log the error for debugging
            \Illuminate\Support\Facades\Log::error('Error fetching product feature details: ' . $e->getMessage());

            // Return a generic error message
            return response()->json([
                'message' => 'An error occurred while fetching product feature details. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        try {
            $request->validate(ProductFeature::$rules);

            $productFeature = $this->model->findOrFail($id);

            // Handle file upload
            if ($request->hasFile('icon')) {
                // Delete the old image if it exists
                if ($productFeature->icon && Storage::disk('public')->exists($productFeature->icon)) {
                    Storage::disk('public')->delete($productFeature->icon);
                }

                // Store the new image
                $productFeatureIconPath = $request->file('icon')->store('product_feature', 'public');
                $productFeature->icon = $productFeatureIconPath;
            }

            // Update product data
            $productFeature->title = $request->title;
            $productFeature->description = $request->description;
            $productFeature->save();

            return response()->json([
                'message' => 'Product feature updated successfully!',
            ], 200); // HTTP 200 OK

        } catch (\Illuminate\Validation\ValidationException $e) {
            return response()->json([
                'message' => 'Validation failed.',
                'errors' => $e->errors(),
            ], 422); // HTTP 422 Unprocessable Entity

        } catch (\Exception $e) {
            Log::error('Error updating product feature: ' . $e->getMessage());
            return response()->json([
                'message' => 'An error occurred while updating the product feature. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        try {
            $product = $this->model->findOrFail($id); // Find the product by ID
            $product->delete(); // Soft delete the product

            return response()->json([
                'message' => 'Product feature deleted successfully!',
            ], 200); // HTTP 200 OK

        } catch (\Exception $e) {
            \Illuminate\Support\Facades\Log::error('Error deleting product feature: ' . $e->getMessage());
            return response()->json([
                'message' => 'An error occurred while deleting the product feature. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }
}
