<?php

namespace App\Http\Controllers\backend;

use App\DataTables\backend\ProductServicesDataTable;
use App\Http\Controllers\Controller;
use App\Models\ProductServices;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class ProductServicesController extends Controller
{
    public function __construct(private ProductServices $model) { }

    /**
     * Display a listing of the resource.
     */
    public function index(ProductServicesDataTable $dataTable)
    {
        $data = ['title' => 'Configure Product Service Information'];
        return $dataTable->render('backend.product_service', $data);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        try {
            $request->validate(ProductServices::$rules);

            // Save to database
            $productService = $this->model->create([
                'title' => $request->title,
                'description' => $request->description
            ]);

            // Return success response
            return response()->json([
                'message' => 'Product service saved successfully!',
                'service' => $productService, // Optionally return the saved product
            ], 200); // HTTP 200 OK

        } catch (\Illuminate\Validation\ValidationException $e) {
            // Handle validation errors
            return response()->json([
                'message' => 'Validation failed.',
                'errors' => $e->errors(),
            ], 422); // HTTP 422 Unprocessable Entity

        } catch (\Exception $e) {
            // Log the error for debugging
            Log::error('Error saving product service: ' . $e->getMessage());

            // Return a generic error message
            return response()->json([
                'message' => 'An error occurred while saving the product service. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        try {
            // Fetch the company data by ID
            $productService = $this->model->findOrFail($id);

            // Return the company data as JSON
            return response()->json($productService);

        } catch (\Exception $e) {
            // Log the error for debugging
            \Illuminate\Support\Facades\Log::error('Error fetching product service details: ' . $e->getMessage());

            // Return a generic error message
            return response()->json([
                'message' => 'An error occurred while fetching product service details. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        try {
            $request->validate(ProductServices::$rules);
            $productService = $this->model->findOrFail($id);

            // Update product data
            $productService->title = $request->title;
            $productService->description = $request->description;
            $productService->save();

            return response()->json([
                'message' => 'Product service updated successfully!',
            ], 200); // HTTP 200 OK

        } catch (\Illuminate\Validation\ValidationException $e) {
            return response()->json([
                'message' => 'Validation failed.',
                'errors' => $e->errors(),
            ], 422); // HTTP 422 Unprocessable Entity

        } catch (\Exception $e) {
            Log::error('Error updating product service: ' . $e->getMessage());
            return response()->json([
                'message' => 'An error occurred while updating the product service. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        try {
            $service = $this->model->findOrFail($id); // Find the product by ID
            $service->delete(); // Soft delete the product

            return response()->json([
                'message' => 'Product service deleted successfully!',
            ], 200); // HTTP 200 OK

        } catch (\Exception $e) {
            \Illuminate\Support\Facades\Log::error('Error deleting product service: ' . $e->getMessage());
            return response()->json([
                'message' => 'An error occurred while deleting the product service. Please try again.',
            ], 500); // HTTP 500 Internal Server Error
        }
    }
}
