<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class Airport extends Model
{
    use HasFactory;

    protected $fillable = [
        'name',
        'country',
        'address',
        'email',
        'latitude',
        'longitude',
        'status',
    ];

    protected $searchable = [
        'name',
        'country',
    ];

    public function formatResponse($airport) : array
    {
        return [
            'name' => $airport->name,
            'country' => $airport->country,
            'address' => $airport->address,
        ];
    }

    public static $rules = [
        'name' => 'required|string|max:150',
        'country' => 'required|string|max:60',
        'address' => 'required|string',
        'email' => 'required|email',
        'latitude' => 'nullable|numeric|between:-90,90',
        'longitude' => 'nullable|numeric|between:-180,180',
        'status' => 'nullable|boolean'
    ];

    /**
     * Scope to find locations within distance
     *
     * Find airports within 50km of a point - example
     * $nearbyAirports = Airport::withinRadius(51.509865, -0.118092, 50)->get();
     *
     */
    public function scopeWithinRadius($query, $lat, $lng, $radius, $unit = 'km')
    {
        $unit = strtolower($unit);
        $earthRadius = ($unit === 'km') ? 6371 : 3959; // km or miles

        return $query->whereRaw("
            $earthRadius * acos(
                cos(radians(?)) *
                cos(radians(latitude)) *
                cos(radians(longitude) - radians(?)) +
                sin(radians(?)) *
                sin(radians(latitude))
            ) <= ?", [$lat, $lng, $lat, $radius]);
    }

    public function scopeInCountry($query, string $country)
    {
        return $query->where('country', $country);
    }
}
